@@ -9,28 +9,27 @@ package Mutex;
 use strict;
 use warnings;
 
-our $VERSION = '1.005';
+our $VERSION = '1.006';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
+use Carp ();
+
 sub new {
     my ($class, %argv) = @_;
+    my $impl = defined($argv{'impl'})
+        ? $argv{'impl'} : defined($argv{'path'}) ? 'Flock' : 'Channel';
 
-    my $pkg = (defined $argv{'impl'})
-        ? $argv{'impl'} : (defined $argv{'path'}) ? 'Flock' : 'Channel';
+    $impl = ucfirst( lc $impl );
 
-    $pkg = ucfirst( lc $pkg );
+    eval "require Mutex::$impl; 1;" ||
+        Carp::croak("Could not load Mutex implementation '$impl': $@");
 
-    if ($INC{"Mutex/$pkg.pm"} || eval "require Mutex::$pkg; 1") {
-        no strict 'refs'; $pkg = 'Mutex::'.$pkg;
+    my $pkg = 'Mutex::'.$impl;
+    no strict 'refs';
 
-        return $pkg->new(%argv);
-    }
-
-    require Carp unless $INC{'Carp.pm'};
-
-    Carp::croak("Could not load Mutex implementation $pkg: $@");
+    return $pkg->new( %argv );
 }
 
 ## base class methods
@@ -44,7 +43,7 @@ sub timedwait {
 
     local $@; local $SIG{'ALRM'} = sub { alarm 0; die "timed out\n" };
 
-    eval { alarm $timeout || 1; $obj->lock_exclusive };
+    eval { alarm $timeout || 1; $obj->lock_exclusive; };
 
     alarm 0;
 
@@ -67,51 +66,51 @@ Mutex - Various locking implementations supporting processes and threads
 
 =head1 VERSION
 
-This document describes Mutex version 1.005
+This document describes Mutex version 1.006
 
 =head1 SYNOPSIS
 
-   use Mutex;
+ use Mutex;
 
-   my $mutex = Mutex->new;
+ my $mutex = Mutex->new;
 
-   {
-       use MCE::Flow max_workers => 4;
+ {
+     use MCE::Flow max_workers => 4;
 
-       mce_flow sub {
-           $mutex->lock;
+     mce_flow sub {
+         $mutex->lock;
 
-           # access shared resource
-           my $wid = MCE->wid; MCE->say($wid); sleep 1;
+         # access shared resource
+         my $wid = MCE->wid; MCE->say($wid); sleep 1;
 
-           $mutex->unlock;
-       };
-   }
+         $mutex->unlock;
+     };
+ }
 
-   {
-       use MCE::Hobo;
+ {
+     use MCE::Hobo;
 
-       MCE::Hobo->create('work', $_) for 1..4;
-       MCE::Hobo->waitall;
-   }
+     MCE::Hobo->create('work', $_) for 1..4;
+     MCE::Hobo->waitall;
+ }
 
-   {
-       use threads;
+ {
+     use threads;
 
-       threads->create('work', $_)   for 5..8;
-       $_->join for ( threads->list );
-   }
+     threads->create('work', $_)   for 5..8;
+     $_->join for ( threads->list );
+ }
 
-   sub work {
-       my ($id) = @_;
-       $mutex->lock;
+ sub work {
+     my ($id) = @_;
+     $mutex->lock;
 
-       # access shared resource
-       print $id, "\n";
-       sleep 1;
+     # access shared resource
+     print $id, "\n";
+     sleep 1;
 
-       $mutex->unlock;
-   }
+     $mutex->unlock;
+ }
 
 =head1 DESCRIPTION
 
@@ -145,17 +144,17 @@ it establishes a C<tempfile> internally including removal on scope exit.
 
 Returns the implementation used for the mutex.
 
-   $m1 = Mutex->new( );
-   $m1->impl();   # Channel
+ $m1 = Mutex->new( );
+ $m1->impl();   # Channel
 
-   $m2 = Mutex->new( path => /tmp/my.lock );
-   $m2->impl();   # Flock
+ $m2 = Mutex->new( path => /tmp/my.lock );
+ $m2->impl();   # Flock
 
-   $m3 = Mutex->new( impl => "Channel" );
-   $m3->impl();   # Channel
+ $m3 = Mutex->new( impl => "Channel" );
+ $m3->impl();   # Channel
 
-   $m4 = Mutex->new( impl => "Flock" );
-   $m4->impl();   # Flock
+ $m4 = Mutex->new( impl => "Flock" );
+ $m4->impl();   # Flock
 
 =head2 $mutex->lock ( void )
 
@@ -167,7 +166,7 @@ locked until mutex->unlock is called.
 
 The method C<lock_exclusive> is an alias for C<lock>.
 
-   ( my $mutex = Mutex->new( path => $0 ) )->lock_exclusive;
+ ( my $mutex = Mutex->new( path => $0 ) )->lock_exclusive;
 
 =head2 $mutex->lock_shared ( void )
 
@@ -186,15 +185,15 @@ automatically.
 Obtains a lock, runs the code block, and releases the lock after the block
 completes. Optionally, the method is C<wantarray> aware.
 
-   my $val = $mutex->synchronize( sub {
-       # access shared resource
-       return 'scalar';
-   });
+ my $val = $mutex->synchronize( sub {
+     # access shared resource
+     return 'scalar';
+ });
 
-   my @ret = $mutex->enter( sub {
-       # access shared resource
-       return @list;
-   });
+ my @ret = $mutex->enter( sub {
+     # access shared resource
+     return @list;
+ });
 
 The method C<enter> is an alias for C<synchronize>.
 
@@ -204,17 +203,17 @@ Blocks until obtaining an exclusive lock. A false value is returned
 if the timeout is reached, and a true value otherwise. The default is
 1 second when omitting timeout.
 
-   my $mutex = Mutex->new( path => $0 );
+ my $mutex = Mutex->new( path => $0 );
 
-   # terminate script if a previous instance is still running
+ # terminate script if a previous instance is still running
 
-   exit unless $mutex->timedwait( 2 );
+ exit unless $mutex->timedwait( 2 );
 
-   ...
+ ...
 
 =head1 REQUIREMENTS
 
-Perl 5.8.0 or later.
+Perl 5.8.1 or later.
 
 =head1 AUTHOR
 
@@ -222,7 +221,7 @@ Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2017 by Mario E. Roy
+Copyright (C) 2017-2020 by Mario E. Roy
 
 Mutex is released under the same license as Perl.
 
