@@ -11,21 +11,20 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized once );
 
-our $VERSION = '1.005';
+our $VERSION = '1.006';
 
 use base 'Mutex';
 use Fcntl ':flock';
 use Carp ();
 
-my $has_threads = $INC{'threads.pm'} ? 1 : 0;
-my $tid = $has_threads ? threads->tid()  : 0;
+my $tid = $INC{'threads.pm'} ? threads->tid() : 0;
 
 sub CLONE {
-    $tid = threads->tid() if $has_threads;
+    $tid = threads->tid() if $INC{'threads.pm'};
 }
 
 sub DESTROY {
-    my ($pid, $obj) = ($has_threads ? $$ .'.'. $tid : $$, @_);
+    my ($pid, $obj) = ($tid ? $$ .'.'. $tid : $$, @_);
 
     $obj->unlock(), close(delete $obj->{_fh}) if $obj->{ $pid };
 
@@ -35,8 +34,7 @@ sub DESTROY {
 }
 
 sub _open {
-    my ($pid, $obj) = ($has_threads ? $$ .'.'. $tid : $$, @_);
-
+    my ($pid, $obj) = ($tid ? $$ .'.'. $tid : $$, @_);
     return if exists $obj->{ $pid };
 
     open $obj->{_fh}, '+>>:raw:stdio', $obj->{path}
@@ -92,7 +90,7 @@ sub new {
             ($tmp_file) = "$tmp_dir/$prog_name.$pid.$tid.$id" =~ /(.*)/;
         }
 
-        $obj{_init} = $has_threads ? $$ .'.'. $tid : $$;
+        $obj{_init} = $tid ? $$ .'.'. $tid : $$;
         $obj{ path} = $tmp_file.'.lock';
 
         # test open
@@ -116,10 +114,10 @@ sub new {
 }
 
 sub lock {
-    my ($pid, $obj) = ($has_threads ? $$ .'.'. $tid : $$, @_);
+    my ($pid, $obj) = ($tid ? $$ .'.'. $tid : $$, shift);
     $obj->_open() unless exists $obj->{ $pid };
 
-    flock ($obj->{_fh}, LOCK_EX), $obj->{ $pid } = 1
+    CORE::flock ($obj->{_fh}, LOCK_EX), $obj->{ $pid } = 1
         unless $obj->{ $pid };
 
     return;
@@ -128,42 +126,41 @@ sub lock {
 *lock_exclusive = \&lock;
 
 sub lock_shared {
-    my ($pid, $obj) = ($has_threads ? $$ .'.'. $tid : $$, @_);
+    my ($pid, $obj) = ($tid ? $$ .'.'. $tid : $$, shift);
     $obj->_open() unless exists $obj->{ $pid };
 
-    flock ($obj->{_fh}, LOCK_SH), $obj->{ $pid } = 1
+    CORE::flock ($obj->{_fh}, LOCK_SH), $obj->{ $pid } = 1
         unless $obj->{ $pid };
 
     return;
 }
 
 sub unlock {
-    my ($pid, $obj) = ($has_threads ? $$ .'.'. $tid : $$, @_);
+    my ($pid, $obj) = ($tid ? $$ .'.'. $tid : $$, shift);
 
-    flock ($obj->{_fh}, LOCK_UN), $obj->{ $pid } = 0
+    CORE::flock ($obj->{_fh}, LOCK_UN), $obj->{ $pid } = 0
         if $obj->{ $pid };
 
     return;
 }
 
 sub synchronize {
-    my ($pid, $obj, $code, @ret) = (
-        $has_threads ? $$ .'.'. $tid : $$, shift, shift
-    );
+    my ($pid, $obj, $code) = ($tid ? $$ .'.'. $tid : $$, shift, shift);
+    my (@ret);
 
-    return if ref($code) ne 'CODE';
+    return unless ref($code) eq 'CODE';
 
     $obj->_open() unless exists $obj->{ $pid };
 
     # lock, run, unlock - inlined for performance
-    flock ($obj->{_fh}, LOCK_EX), $obj->{ $pid } = 1
+    CORE::flock ($obj->{_fh}, LOCK_EX), $obj->{ $pid } = 1
         unless $obj->{ $pid };
 
     (defined wantarray)
       ? @ret = wantarray ? $code->(@_) : scalar $code->(@_)
       : $code->(@_);
 
-    flock ($obj->{_fh}, LOCK_UN), $obj->{ $pid } = 0;
+    CORE::flock ($obj->{_fh}, LOCK_UN), $obj->{ $pid } = 0;
 
     return wantarray ? @ret : $ret[-1];
 }
@@ -186,11 +183,33 @@ Mutex::Flock - Mutex locking via Fcntl
 
 =head1 VERSION
 
-This document describes Mutex::Flock version 1.005
+This document describes Mutex::Flock version 1.006
 
 =head1 DESCRIPTION
 
-A Fcntl implementation for L<Mutex>. See documentation there.
+A Fcntl implementation for C<Mutex>.
+
+The API is described in L<Mutex>.
+
+=over 3
+
+=item new
+
+=item lock
+
+=item lock_exclusive
+
+=item lock_shared
+
+=item unlock
+
+=item synchronize
+
+=item enter
+
+=item timedwait
+
+=back
 
 =head1 AUTHOR
 
